-- -*- Mode: Lua; lua-indent-level: 4 -*-

local function message( message, ... )
    local line = "FlashFind - " .. string.format( message, ... )
    DEFAULT_CHAT_FRAME:AddMessage( line, 1.0, 0.8, 0.4 )
end

local function highlight( pattern )
    local lower = string.lower( pattern )
    local matches = {}
    local found = 0
    for bag = KEYRING_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
        local bagOpen = IsBagOpen( bag )
        for slot = 1, GetContainerNumSlots( bag ) do
            local link = GetContainerItemLink( bag, slot )
            if link then
                if string.find( string.lower( link ), lower, 1, true ) then
                    if not bagOpen and bag ~= BANK_CONTAINER then
                        OpenBag( bag )
                        bagOpen = true
                    end
                    matches[ bag .. "x" .. slot ] = true
                    local _, count = GetContainerItemInfo( bag, slot )
                    found = found + count
                end
            end
        end
    end
    local guildBank = getglobal( "GuildBankFrame" )
    if guildBank and guildBank:IsShown() and guildBank.mode == "bank" then
        local tab = GetCurrentGuildBankTab()
        for slot = 1, MAX_GUILDBANK_SLOTS_PER_TAB do
            local link = GetGuildBankItemLink( tab, slot )
            if link and string.find( string.lower( link ), lower, 1, true ) then
                local index = mod( slot - 1, NUM_SLOTS_PER_GUILDBANK_GROUP ) + 1
                local column = ceil( ( slot - 0.5 ) / NUM_SLOTS_PER_GUILDBANK_GROUP )
                local button = getglobal( "GuildBankColumn" .. column .. "Button" .. index )
                UIFrameFlash( button, 0.2, 0.2, 1.0, true, 0.0, 0.0 )
                local _, count = GetGuildBankItemInfo( tab, slot )
                found = found + count
            end
        end
    end
    if found > 0 then
        for container = 1, NUM_CONTAINER_FRAMES do
            local frame = getglobal( "ContainerFrame" .. container )
            if frame:IsShown() then
                for slot = 1, frame.size do
                    local button = getglobal( frame:GetName() .. "Item" .. slot )
                    if matches[ frame:GetID() .. "x" .. button:GetID() ] then
                        UIFrameFlash( button, 0.2, 0.2, 1.0, true, 0.0, 0.0 )
                    end
                end
            end
        end
        if BankFrame:IsShown() then
            for slot = 1, NUM_BANKGENERIC_SLOTS do
                local button = getglobal( "BankFrameItem" .. slot )
                if matches[ BANK_CONTAINER .. "x" .. button:GetID() ] then
                    UIFrameFlash( button, 0.2, 0.2, 1.0, true, 0.0, 0.0 )
                end
            end
        end
    end
    pattern = string.gsub( pattern, "[%[%]]", "" )
    message( "Found %d %s matching \"%s\".", found, found == 1 and "item" or "items", pattern )
end

local function command( command )
    if command == "" then
        message( "Usage -- \"/find name\" or alt-click an item link or item in your bags or bank." )
    else
        highlight( command )
    end
end

local function linkClicked( link, text, button )
    local _, _, name = string.find( text, "^|c%x+|Hitem.+|h%[(.*)%]" )
    if IsAltKeyDown() and name then
        highlight( name )
    end
end

local function bagItemClicked( button )
    if IsAltKeyDown() then
        local bag = button:GetParent():GetID()
        local slot = button:GetID()
        local link = GetContainerItemLink( bag, slot )
        local _, _, name = string.find( link, "^|c%x+|H.+|h(%[.*%])" )
        highlight( name )
    end
end

local function bankItemClicked( button )
    if IsAltKeyDown() then
        local slot = button:GetID()
        local link = GetContainerItemLink( BANK_CONTAINER, slot )
        local _, _, name = string.find( link, "^|c%x+|H.+|h(%[.*%])" )
        highlight( name )
    end
end

local function guildBankItemClicked( tab, slot )
    if IsAltKeyDown() then
        ClearCursor()
        local link = GetGuildBankItemLink( tab, slot )
        local _, _, name = string.find( link, "^|c%x+|H.+|h(%[.*%])" )
        highlight( name )
    end
end

SLASH_FLASHFIND1 = "/flashfind"
SLASH_FLASHFIND2 = "/find"
SlashCmdList[ "FLASHFIND" ] = command
hooksecurefunc( "SetItemRef", linkClicked )
hooksecurefunc( "ContainerFrameItemButton_OnModifiedClick", bagItemClicked )
hooksecurefunc( "BankFrameItemButtonGeneric_OnModifiedClick", bankItemClicked )
hooksecurefunc( "PickupGuildBankItem", guildBankItemClicked )
